
from .isaleapyear import isaleapyear
from .julianday import date_to_jd


def act_afb(y1: int, m1: int, d1: int,
            y2: int, m2: int, d2: int) -> float:

    if y1 == y2:
        diff_days = date_to_jd(y2, m2, d2) - date_to_jd(y1, m1, d1)
        denom = 366e0 if isaleapyear(y1) and (m1 < 3) else 365e0

        if diff_days < 0:
            raise Exception(
                "Newer date y2-m2-d2 is older than previous date y1-m1-d1.")
        elif diff_days == 0:
            return 0e0
        else:
            return diff_days / denom

    elif y1 < y2:
        diffa = date_to_jd(y1, 12, 31) - date_to_jd(y1, m1, d1)
        denoma = 366e0 if isaleapyear(y1) and (m1 < 3) else 365e0

        diffb = date_to_jd(y2, m2, d2) - date_to_jd(y2, 1, 1)
        denomb = 366e0 if isaleapyear(y2) and (m2 >= 3) else 365e0

        diffy = (y2 - y1 - 1)
        return (diffa / denoma) + (diffb / denomb) + diffy

    else:
        raise Exception(
            "Newer date y2-m2-d2 is older than previous date y1-m1-d1.")



def d30365(y1: int, m1: int, d1: int,
           y2: int, m2: int, d2: int) -> float:
    diff_days = 0
    diff_days += (360 * (y2 - y1))
    diff_days += (30 * (m2 - m1))

    if (d2 == 31) and (d1 >= 30):
        diff_days += 30
    else:
        diff_days += d2

    diff_days -= 30 if d1 > 30 else d1

    if diff_days < 0:
        raise Exception(
            'Newer date y2-m2-d2 is older than previous date y1-m1-d1.')
    elif diff_days == 0:
        return 0e0
    else:
        return diff_days / 365e0



import unittest
from yearfrac import yearfrac
from datetime import datetime
import numpy as np
import pandas as pd


class Test_Yearfrac(unittest.TestCase):

    def test1(self):
        d1 = datetime(2018, 12, 15)
        d2 = datetime(2019, 3, 1)
        x = yearfrac(d1, d2, 'act')
        self.assertEquals(round(x, 8), 0.20547945)

    def test2(self):
        d1 = datetime(2018, 12, 15)
        d2 = datetime(2019, 3, 1)
        x = yearfrac(d1, d2, 'act_isda')
        self.assertEquals(round(x, 8), 0.20547945)

    def test3(self):
        d1 = datetime(2018, 12, 15)
        d2 = datetime(2019, 3, 1)
        x = yearfrac(d1, d2, '30e360')
        self.assertEquals(round(x, 8), 0.21111111)

    def test4(self):
        d1 = datetime(2018, 12, 15)
        d2 = datetime(2019, 3, 1)
        x = yearfrac(d1, d2, '30e360_matu')
        self.assertEquals(round(x, 8), 0.21111111)

    def test_vector_vector(self):
        v1 = [datetime(2018, 12, 15), datetime(2018, 12, 15)]
        v2 = [datetime(2019, 3, 1), datetime(2019, 3, 1)]
        x = yearfrac(v1, v2, 'act')
        self.assertEquals(
            [round(x[0], 8), round(x[1], 8)],
            [0.20547945, 0.20547945]
        )

    def test_vector_scalar(self):
        v = [datetime(2018, 12, 15), datetime(2018, 12, 15)]
        s = datetime(2019, 3, 1)
        x = yearfrac(v, s, 'act')
        self.assertEquals(
            [round(x[0], 8), round(x[1], 8)],
            [0.20547945, 0.20547945]
        )

    def test_scalar_vector(self):
        s = datetime(2018, 12, 15)
        v = [datetime(2019, 3, 1), datetime(2019, 3, 1)]
        x = yearfrac(s, v, 'act')
        self.assertEquals(
            [round(x[0], 8), round(x[1], 8)],
            [0.20547945, 0.20547945]
        )

    def test_nparray_vector(self):
        v1 = np.array(
            [datetime(2018, 12, 15), datetime(2018, 12, 15)]
        )
        v2 = [datetime(2019, 3, 1), datetime(2019, 3, 1)]
        x = yearfrac(v1, v2, 'act')
        self.assertEquals(
            [round(x[0], 8), round(x[1], 8)],
            [0.20547945, 0.20547945]
        )

    def test_pdseries_vector(self):
        v1 = pd.Series(
            [datetime(2018, 12, 15), datetime(2018, 12, 15)]
        )
        v2 = [datetime(2019, 3, 1), datetime(2019, 3, 1)]
        x = yearfrac(v1, v2, 'act')
        self.assertEquals(
            [round(x[0], 8), round(x[1], 8)],
            [0.20547945, 0.20547945]
        )


# run
if __name__ == '__main__':
    unittest.main()



import unittest
from yearfrac import d30365


class Test_ActIsda(unittest.TestCase):

    def test1(self):
        x = d30365(2018, 12, 15, 2019, 3, 1)
        self.assertEquals(round(x, 8), 0.20821918)


# run
if __name__ == '__main__':
    unittest.main()



import unittest
from yearfrac import act_afb


class Test_ActAfb(unittest.TestCase):

    def test1(self):
        x = act_afb(2018, 12, 15, 2019, 3, 1)
        self.assertEquals(round(x, 8), 0.20547945)


# run
if __name__ == '__main__':
    unittest.main()



from collections.abc import Iterable
from .method_factory import methods
import datetime
import numpy as np


def __yearfrac_scalar_scalar(s1: datetime.datetime,
                             s2: datetime.datetime,
                             method_function) -> float:
    return method_function(
        s1.year, s1.month, s1.day,
        s2.year, s2.month, s2.day
    )


def __yearfrac_scalar_vector(s: datetime.datetime,
                             v: np.ndarray,
                             method_function) -> list:
    sv = [s] * len(v)
    __yearfrac_vector_vector(sv, v, method_function)


def __yearfrac_vector_scalar(v: np.ndarray,
                             s: datetime.datetime,
                             method_function) -> list:
    sv = [s] * len(v)
    __yearfrac_vector_vector(v, sv, method_function)


def __yearfrac_vector_vector(v1: np.ndarray,
                             v2: np.ndarray,
                             method_function) -> list:
    rows = zip(v1, v2)
    return [
        method_function(
            s1.year, s1.month, s1.day,
            s2.year, s2.month, s2.day
        )
        for s1, s2 in rows
    ]


__algorithm_mapper = {
    (True, True): __yearfrac_vector_vector,
    (False, False): __yearfrac_scalar_scalar,
    (True, False):
        lambda v, s, m: __yearfrac_vector_vector(v, [s] * len(v), m),
    (False, True):
        lambda s, v, m: __yearfrac_vector_vector([s] * len(v), v, m)
}


# output is list or float
def yearfrac(d1: datetime.datetime,
             d2: datetime.datetime,
             method: str = 'act'):
    yearfrac_func = __algorithm_mapper[
        (
            isinstance(d1, Iterable),
            isinstance(d2, Iterable)
        )
    ]
    method_function = methods[method]
    return yearfrac_func(d1, d2, method_function)



from .act_isda import act_isda
from .act_afb import act_afb
from .d30360e import d30360e
from .d30365 import d30365

# Add new method calls here.
# The default signature is function_name(x1, x2, x3, y1, y2, y3).
# If there are more input arguments, use lambda expressions
# or define a function wrapper.

methods = dict.fromkeys(['act', 'act_afb', 'afb'], act_afb)
methods.update(
    dict.fromkeys(
        ['d30360e', '30e360'],
        lambda x1, x2, x3, y1, y2, y3: d30360e(
            x1, x2, x3,
            y1, y2, y3,
            False
        )
    )
)
methods.update({'act_isda': act_isda})
methods.update(
    dict.fromkeys(
        ['d30360e_matu', '30e360_matu'],
        lambda x1, x2, x3, y1, y2, y3: d30360e(
            x1, x2, x3,
            y1, y2, y3,
            True
        )
    )
)
methods.update({'30365': d30365})




def isaleapyear(y: int) -> bool:
    if not isinstance(y, int):
        raise Exception("y is not an integer data type.")
    if y < 0:
        raise Exception("y is negative")

    return not(y % 4) and (bool(y % 100) or not(y % 400))
